import pysoem
import threading
import time
import ctypes
import struct
import tkinter as tk
from tkinter import messagebox

# EtherCAT 마스터 생성
master = pysoem.Master()

master.open('\\Device\\NPF_{60FC8210-8A23-4375-B5F6-31D0E27AB1ED}')

# 슬레이브 초기화
if master.config_init() > 0:
    print(f'슬레이브 {len(master.slaves)}개 검색됨!')
else:
    print('No slaves found')
    master.close()
    raise Exception("NO SLAVE")

slave = master.slaves[0] #마스터랑 먼쪽
slave1 = master.slaves[1] #마스터랑 가까운쪽

master.config_map() #슬레이브가 pre-op로 전환됨

if master.state_check(pysoem.SAFEOP_STATE, timeout=50_000) != pysoem.SAFEOP_STATE:
    master.close()
    raise Exception("not all slaves reached SAFEOP state")
        
slave.dc_sync(act=True, sync0_cycle_time=10_000_000)
slave1.dc_sync(act=True, sync0_cycle_time=10_000_000)
master.state = pysoem.OP_STATE

master.send_processdata()
master.receive_processdata(timeout=2000)

# request OP state for all slaves
master.write_state()
all_slaves_reached_op_state = False
for i in range(40):
    master.state_check(pysoem.OP_STATE, timeout=50_000)
    if master.state == pysoem.OP_STATE:
        all_slaves_reached_op_state = True
        break

if all_slaves_reached_op_state:
    print("모든 슬레이브 활성상태")


def update_scrollbar_value():
    # Scrollbar의 현재 값 가져오기
    value = scrollbar.get() #0~180
    # Scrollbar의 값을 첫 번째 라벨에 표시
    label2.config(text=f"Scrollbar Value: {value}")
    tmp1[0] = value
    slave.output = bytes(tmp1)
    master.send_processdata() #update
    print(f"스크롤1={value}")

def update_scrollbar_value2():
    # Scrollbar의 현재 값 가져오기
    value = scrollbar2.get() #0~180
    # Scrollbar의 값을 첫 번째 라벨에 표시
    label3.config(text=f"Scrollbar Value: {value}")
    tmp2[0] = value
    slave1.output = bytes(tmp2)
    master.send_processdata() #update
    print(f"스크롤2={value}")

#OUTPUT PDO의 길이만큼 버퍼를 생성함
output_len1 = len(slave.output) #32
tmp1 = bytearray([0 for i in range(output_len1)])
output_len2 = len(slave1.output) #32
tmp2 = bytearray([0 for i in range(output_len2)])

# GUI 설정
root = tk.Tk()
root.title("녹칸다의 이더캣통신 프로그램!")

root.geometry("600x400")

# Scrollbar 위젯 생성 (0에서 180까지의 값 설정)
scrollbar = tk.Scale(root, from_=0, to=180, orient="horizontal", command=lambda v: update_scrollbar_value())
scrollbar.pack(pady=10)

# Label 1 생성 (Scrollbar의 값을 표시)
label2 = tk.Label(root, text="Scrollbar Value: 0", font=("Arial", 14))
label2.pack(pady=10)

# Scrollbar 위젯 생성 (0에서 180까지의 값 설정)
scrollbar2 = tk.Scale(root, from_=0, to=180, orient="horizontal", command=lambda v: update_scrollbar_value2())
scrollbar2.pack(pady=10)

# Label 1 생성 (Scrollbar의 값을 표시)
label3 = tk.Label(root, text="Scrollbar Value: 0", font=("Arial", 14))
label3.pack(pady=10)

root.mainloop()

print("프로그램이 종료되었습니다.")
master.state = pysoem.INIT_STATE
# request INIT state for all slaves
master.write_state()
master.close()
